//
//  AASQueuedObjectOperation.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/12/08.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* AASQueuedObjectOperation is a subclass of NSOperation that has some basic support for an input queue (into which you can load objects for your worker thread to work on, in a thread-safe manner) and an output array (from which you can read your results in a thread-safe manner). It is used as a basis for the AASFileLoadOperation class, but can be useful as the basis for any number of similar operations. Like NSOperation, AASQueuedObjectOperation cannot be used on its own, it must be subclassed. If the subclass must access one of the arrays directly, use @synchronized to lock on the variable. */

#import <Cocoa/Cocoa.h>



#pragma mark Protocol


@protocol AASQueuedObjectOperationDelegateProtocol <NSObject>

- (void)queuedObjectOperationPostedResults:(id)operation;
- (void)queuedObjectOperationComplete:(id)operation;

@end




#pragma mark -


@interface AASQueuedObjectOperation : NSOperation {
	NSMutableArray *inQueue;
	NSMutableArray *outResults;
	
	float updateFrequency;
	
	id currentObject;
	
	NSDate *lastDelegateMessageDate;
	
	NSRange latestResultsRange;
	
	NSArray *runLoopModes;
	
	id delegate;
}



#pragma mark -
#pragma mark Setup


@property(assign) id<AASQueuedObjectOperationDelegateProtocol> delegate;

// The minimum time between notifications to the delegate. Defaults to 0.1 seconds.
@property(assign) float updateFrequency;

// The run loop modes on which the delegate wishes to be notified. Defaults to just NSDefaultRunLoopMode.
@property(copy) NSArray *runLoopModes;



#pragma mark -
#pragma mark Modifiying Queue


// Allows duplicate objects in queue
- (void)addObjectToQueue:(id)newObject;
- (void)addObjectsToQueue:(NSArray *)newObjects;
- (void)addObjectToFrontOfQueue:(id)newObject;
- (void)addObjectsToFrontOfQueue:(NSArray *)newObjects;

// Get an autoreleased copy of the queue, or set the queue's contents (replaces anything else that was in the queue previously).
@property(copy) NSArray *queue;

// Only removes the first instance of each object. Returns yes if the object was found.
- (BOOL)removeObjectFromQueue:(id)object;
// Only removes the first instance of each object. Returns yes if any object was found.
- (BOOL)removeObjectsFromQueue:(NSArray *)objects;
// Removes everything in the queue
- (void)clearQueue;




#pragma mark -
#pragma mark Using Queue

// Returns the next object and removes it from the queue. Use in the main method of your subclass.
- (id)popFromQueue;

@property(readonly) NSUInteger queueCount;

// Returns the last object popped from the queue. Thread-safe.
@property(readonly) id currentObject;




#pragma mark -
#pragma mark Posting Results

// In your subclass's main method, call these to add results to the queue. The delegate autormatically gets informed of the update on the main thread.
- (void)postResult:(id)result;
- (void)postResults:(NSArray *)resultsArray;




#pragma mark -
#pragma mark Returning Results

// Returns an array of results that have been added since the last call to latestResults. Returns nil if no new results.
@property(readonly) NSArray *latestResults;

// Returns all results available up to this point.
@property(readonly) NSArray *results;

@property(readonly) NSUInteger resultsCount;





@end
